//! The ActivityPuppy social media server.
//!
//! This crate contains the implementation of the ActivityPuppy's server binary. Also see the library,
//! [`puppy`], and the other two major components: [`store`] for persistence and [`fetch`] for the
//! federation implementation.
//!
//! [`store`]: puppy::store
//! [`fetch`]: puppy::fetch
#![feature(try_blocks, yeet_expr)]

use puppy::{config::Config, Context};
use tracing::Level;
use tracing_forest::ForestLayer;
use tracing_subscriber::{
    filter::filter_fn, layer::SubscriberExt as _, util::SubscriberInitExt as _, Registry,
};

mod sig;
mod api;

/// Starts up the whole shebang.
#[tokio::main]
async fn main() {
    Registry::default()
        .with(filter_fn(|meta| !meta.target().starts_with("reqwest")))
        .with(filter_fn(|meta| *meta.level() < Level::DEBUG))
        .with(ForestLayer::default())
        .init();
    // TODO: load the config from a file or something.
    let config = Config {
        ap_domain: "test.piss-on.me".to_string(),
        wf_domain: "test.piss-on.me".to_string(),
        state_dir: ".state".to_string(),
        port: 1312,
    };
    let context = Context::load(config).unwrap();
    // Start the web server
    api::start(context).await.unwrap();
}
